@@ -1,6 +1,6 @@
 use crate::document::AcquiredNodes;
 use crate::drawing_ctx::DrawingCtx;
-use crate::element::SetAttributes;
+use crate::element::ElementTrait;
 use crate::node::{CascadedValues, Node};
 use crate::paint_server::resolve_color;
 use crate::rect::IRect;
@@ -24,7 +24,7 @@ pub struct Flood {
     pub color: cssparser::RGBA,
 }
 
-impl SetAttributes for FeFlood {
+impl ElementTrait for FeFlood {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         self.base.parse_no_inputs(attrs, session);
     }
