@@ -7,15 +7,14 @@ use markup5ever::{
 
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId, NodeStack};
-use crate::drawing_ctx::ViewParams;
-use crate::element::{set_attribute, Draw, Element, SetAttributes};
+use crate::drawing_ctx::Viewport;
+use crate::element::{set_attribute, ElementData, ElementTrait};
 use crate::error::*;
 use crate::href::{is_href, set_href};
 use crate::length::*;
 use crate::node::{CascadedValues, Node, NodeBorrow};
 use crate::paint_server::resolve_color;
 use crate::parsers::{Parse, ParseValue};
-use crate::properties::ComputedValues;
 use crate::rect::{rect_to_transform, Rect};
 use crate::session::Session;
 use crate::transform::{Transform, TransformAttribute};
@@ -65,7 +64,7 @@ pub struct Stop {
      * they go into property_defs.rs */
 }
 
-impl SetAttributes for Stop {
+impl ElementTrait for Stop {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             if attr.expanded() == expanded_name!("", "offset") {
@@ -75,8 +74,6 @@ impl SetAttributes for Stop {
     }
 }
 
-impl Draw for Stop {}
-
 /// Parameters specific to each gradient type, before being resolved.
 /// These will be composed together with UnreseolvedVariant from fallback
 /// nodes (referenced with e.g. <linearGradient xlink:href="#fallback">) to form
@@ -410,14 +407,12 @@ impl UnresolvedGradient {
     /// and adds their info to the UnresolvedGradient &self.
     fn add_color_stops_from_node(&mut self, node: &Node, opacity: UnitInterval) {
         assert!(matches!(
-            *node.borrow_element(),
-            Element::LinearGradient(_) | Element::RadialGradient(_)
+            *node.borrow_element_data(),
+            ElementData::LinearGradient(_) | ElementData::RadialGradient(_)
         ));
 
         for child in node.children().filter(|c| c.is_element()) {
-            let elt = child.borrow_element();
-
-            if let Element::Stop(ref stop) = *elt {
+            if let ElementData::Stop(ref stop) = &*child.borrow_element_data() {
                 let cascaded = CascadedValues::new_from_node(&child);
                 let values = cascaded.get();
 
@@ -511,7 +506,7 @@ impl RadialGradient {
     }
 }
 
-impl SetAttributes for Common {
+impl Common {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
@@ -539,7 +534,7 @@ impl SetAttributes for Common {
     }
 }
 
-impl SetAttributes for LinearGradient {
+impl ElementTrait for LinearGradient {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         self.common.set_attributes(attrs, session);
 
@@ -556,8 +551,6 @@ impl SetAttributes for LinearGradient {
     }
 }
 
-impl Draw for LinearGradient {}
-
 macro_rules! impl_gradient {
     ($gradient_type:ident, $other_type:ident) => {
         impl $gradient_type {
@@ -600,11 +593,11 @@ macro_rules! impl_gradient {
                             return Err(AcquireError::CircularReference(acquired_node.clone()));
                         }
 
-                        let unresolved = match *acquired_node.borrow_element() {
-                            Element::$gradient_type(ref g) => {
+                        let unresolved = match *acquired_node.borrow_element_data() {
+                            ElementData::$gradient_type(ref g) => {
                                 g.get_unresolved(&acquired_node, opacity)
                             }
-                            Element::$other_type(ref g) => {
+                            ElementData::$other_type(ref g) => {
                                 g.get_unresolved(&acquired_node, opacity)
                             }
                             _ => return Err(AcquireError::InvalidLinkType(node_id.clone())),
@@ -629,7 +622,7 @@ macro_rules! impl_gradient {
 impl_gradient!(LinearGradient, RadialGradient);
 impl_gradient!(RadialGradient, LinearGradient);
 
-impl SetAttributes for RadialGradient {
+impl ElementTrait for RadialGradient {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         self.common.set_attributes(attrs, session);
 
@@ -657,19 +650,17 @@ impl SetAttributes for RadialGradient {
     }
 }
 
-impl Draw for RadialGradient {}
-
 impl ResolvedGradient {
     pub fn to_user_space(
         &self,
         object_bbox: &Option<Rect>,
-        current_params: &ViewParams,
-        values: &ComputedValues,
+        viewport: &Viewport,
+        values: &NormalizeValues,
     ) -> Option<UserSpaceGradient> {
         let units = self.units.0;
         let transform = rect_to_transform(object_bbox, units).ok()?;
-        let view_params = current_params.with_units(units);
-        let params = NormalizeParams::new(values, &view_params);
+        let view_params = viewport.with_units(units);
+        let params = NormalizeParams::from_values(values, &view_params);
 
         let gradient_transform = self.transform.to_transform();
         let transform = transform.pre_transform(&gradient_transform).invert()?;
