@@ -1153,6 +1153,20 @@ make_property!(
     "plaintext" => Plaintext,
 );
 
+make_property!(
+    /// `vector-effect` property.
+    ///
+    /// SVG2: <https://svgwg.org/svg2-draft/coords.html#VectorEffectProperty>
+    VectorEffect,
+    default: None,
+    inherits_automatically: false,
+
+    identifiers:
+    "none" => None,
+    "non-scaling-stroke" => NonScalingStroke,
+    // non-scaling-size, non-rotation, fixed-position not implemented
+);
+
 make_property!(
     /// `visibility` property.
     ///
