@@ -6,8 +6,8 @@ use crate::aspect_ratio::*;
 use crate::bbox::BoundingBox;
 use crate::coord_units::CoordUnits;
 use crate::document::{AcquiredNodes, NodeId};
-use crate::drawing_ctx::{ClipMode, DrawingCtx, ViewParams};
-use crate::element::{set_attribute, Draw, Element, SetAttributes};
+use crate::drawing_ctx::{ClipMode, DrawingCtx, Viewport};
+use crate::element::{set_attribute, ElementData, ElementTrait};
 use crate::error::*;
 use crate::href::{is_href, set_href};
 use crate::layout::StackingContext;
@@ -23,14 +23,13 @@ use crate::xml::Attributes;
 #[derive(Default)]
 pub struct Group();
 
-impl SetAttributes for Group {}
-
-impl Draw for Group {
+impl ElementTrait for Group {
     fn draw(
         &self,
         node: &Node,
         acquired_nodes: &mut AcquiredNodes<'_>,
         cascaded: &CascadedValues<'_>,
+        viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
@@ -48,10 +47,10 @@ impl Draw for Group {
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
-            values,
+            viewport,
             clipping,
             None,
-            &mut |an, dc, _transform| node.draw_children(an, cascaded, dc, clipping),
+            &mut |an, dc| node.draw_children(an, cascaded, viewport, dc, clipping),
         )
     }
 }
@@ -63,21 +62,19 @@ impl Draw for Group {
 #[derive(Default)]
 pub struct NonRendering;
 
-impl SetAttributes for NonRendering {}
-
-impl Draw for NonRendering {}
+impl ElementTrait for NonRendering {}
 
+/// The `<switch>` element.
 #[derive(Default)]
 pub struct Switch();
 
-impl SetAttributes for Switch {}
-
-impl Draw for Switch {
+impl ElementTrait for Switch {
     fn draw(
         &self,
         node: &Node,
         acquired_nodes: &mut AcquiredNodes<'_>,
         cascaded: &CascadedValues<'_>,
+        viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
@@ -95,10 +92,10 @@ impl Draw for Switch {
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
-            values,
+            viewport,
             clipping,
             None,
-            &mut |an, dc, _transform| {
+            &mut |an, dc| {
                 if let Some(child) = node.children().filter(|c| c.is_element()).find(|c| {
                     let elt = c.borrow_element();
                     elt.get_cond(dc.user_language())
@@ -106,6 +103,7 @@ impl Draw for Switch {
                     child.draw(
                         an,
                         &CascadedValues::clone_with_node(cascaded, &child),
+                        viewport,
                         dc,
                         clipping,
                     )
@@ -221,16 +219,16 @@ impl Svg {
         self.preserve_aspect_ratio
     }
 
-    fn push_viewport(
+    fn make_svg_viewport(
         &self,
         node: &Node,
         cascaded: &CascadedValues<'_>,
+        current_viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
-    ) -> Option<ViewParams> {
+    ) -> Option<Viewport> {
         let values = cascaded.get();
 
-        let view_params = draw_ctx.get_view_params();
-        let params = NormalizeParams::new(values, &view_params);
+        let params = NormalizeParams::new(values, current_viewport);
 
         let has_parent = node.parent().is_some();
 
@@ -269,11 +267,17 @@ impl Svg {
             )
         };
 
-        draw_ctx.push_new_viewport(vbox, viewport, self.preserve_aspect_ratio, clip_mode)
+        draw_ctx.push_new_viewport(
+            current_viewport,
+            vbox,
+            viewport,
+            self.preserve_aspect_ratio,
+            clip_mode,
+        )
     }
 }
 
-impl SetAttributes for Svg {
+impl ElementTrait for Svg {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
@@ -287,14 +291,13 @@ impl SetAttributes for Svg {
             }
         }
     }
-}
 
-impl Draw for Svg {
     fn draw(
         &self,
         node: &Node,
         acquired_nodes: &mut AcquiredNodes<'_>,
         cascaded: &CascadedValues<'_>,
+        viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
@@ -312,17 +315,21 @@ impl Draw for Svg {
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
-            values,
+            viewport, // FIXME: should this be the svg_viewport from below?
             clipping,
             None,
-            &mut |an, dc, _transform| {
-                let _params = self.push_viewport(node, cascaded, dc);
-                node.draw_children(an, cascaded, dc, clipping)
+            &mut |an, dc| {
+                if let Some(svg_viewport) = self.make_svg_viewport(node, cascaded, viewport, dc) {
+                    node.draw_children(an, cascaded, &svg_viewport, dc, clipping)
+                } else {
+                    Ok(dc.empty_bbox())
+                }
             },
         )
     }
 }
 
+/// The `<use>` element.
 pub struct Use {
     link: Option<NodeId>,
     x: Length<Horizontal>,
@@ -354,7 +361,7 @@ impl Default for Use {
     }
 }
 
-impl SetAttributes for Use {
+impl ElementTrait for Use {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
@@ -379,21 +386,19 @@ impl SetAttributes for Use {
             }
         }
     }
-}
 
-impl Draw for Use {
     fn draw(
         &self,
         node: &Node,
         acquired_nodes: &mut AcquiredNodes<'_>,
         cascaded: &CascadedValues<'_>,
+        viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         if let Some(link) = self.link.as_ref() {
             let values = cascaded.get();
-            let view_params = draw_ctx.get_view_params();
-            let params = NormalizeParams::new(values, &view_params);
+            let params = NormalizeParams::new(values, viewport);
             let rect = self.get_rect(&params);
 
             let stroke_paint = values.stroke().0.resolve(
@@ -421,6 +426,7 @@ impl Draw for Use {
                 rect,
                 link,
                 clipping,
+                viewport,
                 fill_paint,
                 stroke_paint,
             )
@@ -430,6 +436,7 @@ impl Draw for Use {
     }
 }
 
+/// The `<symbol>` element.
 #[derive(Default)]
 pub struct Symbol {
     preserve_aspect_ratio: AspectRatio,
@@ -446,7 +453,7 @@ impl Symbol {
     }
 }
 
-impl SetAttributes for Symbol {
+impl ElementTrait for Symbol {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
@@ -462,10 +469,9 @@ impl SetAttributes for Symbol {
     }
 }
 
-impl Draw for Symbol {}
-
 coord_units!(ClipPathUnits, CoordUnits::UserSpaceOnUse);
 
+/// The `<clipPath>` element.
 #[derive(Default)]
 pub struct ClipPath {
     units: ClipPathUnits,
@@ -477,7 +483,7 @@ impl ClipPath {
     }
 }
 
-impl SetAttributes for ClipPath {
+impl ElementTrait for ClipPath {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             if attr.expanded() == expanded_name!("", "clipPathUnits") {
@@ -487,11 +493,10 @@ impl SetAttributes for ClipPath {
     }
 }
 
-impl Draw for ClipPath {}
-
 coord_units!(MaskUnits, CoordUnits::ObjectBoundingBox);
 coord_units!(MaskContentUnits, CoordUnits::UserSpaceOnUse);
 
+/// The `<mask>` element.
 pub struct Mask {
     x: Length<Horizontal>,
     y: Length<Vertical>,
@@ -536,7 +541,7 @@ impl Mask {
     }
 }
 
-impl SetAttributes for Mask {
+impl ElementTrait for Mask {
     fn set_attributes(&mut self, attrs: &Attributes, session: &Session) {
         for (attr, value) in attrs.iter() {
             match attr.expanded() {
@@ -560,14 +565,13 @@ impl SetAttributes for Mask {
     }
 }
 
-impl Draw for Mask {}
-
+/// The `<a>` element.
 #[derive(Default)]
 pub struct Link {
     pub link: Option<String>,
 }
 
-impl SetAttributes for Link {
+impl ElementTrait for Link {
     fn set_attributes(&mut self, attrs: &Attributes, _session: &Session) {
         for (attr, value) in attrs.iter() {
             let expanded = attr.expanded();
@@ -576,21 +580,20 @@ impl SetAttributes for Link {
             }
         }
     }
-}
 
-impl Draw for Link {
     fn draw(
         &self,
         node: &Node,
         acquired_nodes: &mut AcquiredNodes<'_>,
         cascaded: &CascadedValues<'_>,
+        viewport: &Viewport,
         draw_ctx: &mut DrawingCtx,
         clipping: bool,
     ) -> Result<BoundingBox, RenderingError> {
         // If this element is inside of <text>, do not draw it.
         // The <text> takes care of it.
         for an in node.ancestors() {
-            if matches!(&*an.borrow_element(), Element::Text(_)) {
+            if matches!(&*an.borrow_element_data(), ElementData::Text(_)) {
                 return Ok(draw_ctx.empty_bbox());
             }
         }
@@ -620,10 +623,10 @@ impl Draw for Link {
         draw_ctx.with_discrete_layer(
             &stacking_ctx,
             acquired_nodes,
-            values,
+            viewport,
             clipping,
             None,
-            &mut |an, dc, _transform| node.draw_children(an, &cascaded, dc, clipping),
+            &mut |an, dc| node.draw_children(an, &cascaded, viewport, dc, clipping),
         )
     }
 }
